(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const injectorMap = new Map();
    function getInjector(type) {
        return injectorMap.get(type);
    }
    exports.getInjector = getInjector;
    function hasInjector(type) {
        return injectorMap.has(type);
    }
    exports.hasInjector = hasInjector;
    function createInjector(type, initial) {
        if (!injectorMap.has(type)) {
            injectorMap.set(type, new Injector(type, initial));
        }
        return injectorMap.get(type);
    }
    exports.createInjector = createInjector;
    class Injector {
        constructor(id, initial) {
            this.id = id;
            this.factoryMap = new Map();
            if (initial) {
                for (let [name, factory] of initial) {
                    this.register(name, factory);
                }
            }
        }
        get(name) {
            return this.factoryMap.get(name);
        }
        register(name, factory) {
            if (this.factoryMap.has(name)) {
                throw new Error(`${name} already exists on the ${this.id} factory`);
            }
            this.factoryMap.set(name, factory);
        }
    }
    exports.Injector = Injector;
});
//# sourceMappingURL=injector.js.map