var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "fs", "path", "../HttpError", "../log", "../processors/before/file.processor", "mkdirp"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const fs_1 = require("fs");
    const path_1 = require("path");
    const HttpError_1 = require("../HttpError");
    const log_1 = require("../log");
    const file_processor_1 = require("../processors/before/file.processor");
    const mkdirp = require("mkdirp");
    exports.saveFiles = ({ allowOverwrite = false, createUploadDirectory = false, directory }) => {
        const uploadDirectory = path_1.resolve(directory);
        if (createUploadDirectory) {
            mkdirp.sync(uploadDirectory);
        }
        return (request) => __awaiter(void 0, void 0, void 0, function* () {
            var e_1, _a;
            if (request.method !== 'POST') {
                throw new HttpError_1.HttpError(400 /* BadRequest */);
            }
            let files = [];
            let failed = [];
            if (file_processor_1.hasIncomingFiles(request)) {
                try {
                    for (var _b = __asyncValues(request.files()), _c; _c = yield _b.next(), !_c.done;) {
                        let { file, filename } = _c.value;
                        const absoluteFilename = path_1.resolve(path_1.join(directory, path_1.basename(filename)));
                        if (allowOverwrite || !fs_1.existsSync(absoluteFilename)) {
                            file.pipe(fs_1.createWriteStream(absoluteFilename));
                            log_1.log.info(`Saved ${filename} to ${absoluteFilename}`);
                            files.push(filename);
                        }
                        else {
                            file.resume();
                            log_1.log.warn(`File already exists! ${absoluteFilename}`);
                            failed.push(filename);
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            else {
                log_1.log.warn('request contains no files property. Was the file processor applied?');
            }
            return {
                files,
                failed
            };
        });
    };
});
//# sourceMappingURL=saveFiles.js.map