var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./HttpError"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const HttpError_1 = require("./HttpError");
    function isRoute(value) {
        return typeof value === 'object' && typeof value.test === 'function' && typeof value.run === 'function';
    }
    exports.isRoute = isRoute;
    exports.route = ({ before = [], guards = [], middleware: action, transforms = [], after = [], errorHandler }) => {
        const middleware = Array.isArray(action) ? exports.multiroute(action) : action;
        function test(request, response) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    for (let process of before) {
                        yield process(request, response);
                    }
                    return guards.every((guard) => guard(request));
                }
                catch (e) {
                    if (errorHandler) {
                        errorHandler(e, response);
                        return false;
                    }
                    throw e;
                }
            });
        }
        function run(request, response) {
            return __awaiter(this, void 0, void 0, function* () {
                let result;
                try {
                    result = yield middleware(request, response);
                    for (let transform of transforms) {
                        if (response.finished) {
                            break;
                        }
                        transform(result, request, response);
                    }
                    for (let process of after) {
                        yield process(request, response);
                    }
                    return result;
                }
                catch (e) {
                    if (errorHandler) {
                        errorHandler(e, response, result);
                    }
                    else {
                        throw e;
                    }
                }
            });
        }
        return {
            test,
            run
        };
    };
    exports.multiroute = (list) => {
        const routes = list.map((item) => (isRoute(item) ? item : exports.route(item)));
        const middleware = (request, response) => __awaiter(void 0, void 0, void 0, function* () {
            for (let route of routes) {
                if (yield route.test(request, response)) {
                    return yield route.run(request, response);
                }
            }
            throw new HttpError_1.HttpError(404 /* NotFound */);
        });
        return middleware;
    };
});
//# sourceMappingURL=route.js.map